/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "argList.H"
#include "IOmanip.H"
#include "ODE.H"
#include "ODESolver.H"
#include "RK.H"

using namespace Foam;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

class testODE
:
    public ODE
{

public:

    testODE()
    {}

    label nEqns() const
    {
       
       return 2;  
   }

    void derivatives
    (
        const scalar x,
        const scalarField& y,
        scalarField& dydx
    ) const
    {
        dydx[0] = y[1];
        dydx[1] = y[0];
        
    }

    void jacobian
    (
        const scalar x,
        const scalarField& y,
        scalarField& dfdx,
        Matrix<scalar>& dfdy
    ) const
    {
        dfdx[0] = 0.0;
        dfdx[0] = 0.0;

        dfdy[0][0] = 0.0;
        dfdy[0][1] = 1.0;
        dfdy[1][0] = 1.0;
        dfdy[1][1] = 0.0;

        
    }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Main program:

int main(int argc, char *argv[])
{
    argList::validArgs.clear();
    argList::validArgs.append("ODESolver");
    argList args(argc, argv);

    word ODESolverName(args.additionalArgs()[0]);

    testODE ode;
    autoPtr<ODESolver> odeSolver = ODESolver::New(ODESolverName, ode);

    scalar xStart = 0.000001;
    scalarField yStart(ode.nEqns());
    
    yStart[0] = 0.0;
    yStart[1] = 1.0;
    
    scalarField dyStart(ode.nEqns());
    ode.derivatives(xStart, yStart, dyStart);

    scalar x = xStart;	
    scalar xEnd =  1.0;
    scalarField y = yStart;


    
    scalar hEst = 0.06;

    odeSolver->solve(ode, x, xEnd, y, 1e-4, hEst);


    Info      << "Numerical:  y(2.0) = " << y << ", hEst = " << hEst << endl;

    Info << "\nEnd\n" << endl;

    return 0;
}


// ************************************************************************* //
